Class: Autobuild::Git
Defined Under Namespace
Classes: NetworkAccessNeeded, Status
Class Attribute Summary collapse
-
.default_alternates ⇒ Array
A default list of repositories that should be used as reference repositories for all Git importers.
Instance Attribute Summary collapse
-
#additional_remotes ⇒ (String,String,String)
readonly
A list of remotes that should be set up in the git config.
-
#alternates ⇒ Array<String>
A list of local (same-host) repositories that will be used instead of the remote one when possible.
-
#branch ⇒ Object
The branch this importer is tracking.
-
#commit ⇒ Object
The commit we are pointing to.
-
#local_branch ⇒ Object
The branch that should be used on the local clone.
-
#merge ⇒ Object
writeonly
Set the merge flag.
-
#push_to ⇒ Object
If set, this URL will be listed as a pushurl for the tracked branch.
-
#remote_branch ⇒ Object
The remote branch to which we should push.
-
#remote_name ⇒ Object
The name of the remote that should be set up by the importer.
-
#repository ⇒ Object
The remote repository URL.
-
#single_branch ⇒ Object
writeonly
Set the #single_branch? predicate.
-
#tag ⇒ Object
The tag we are pointing to.
-
#with_submodules ⇒ Object
writeonly
Set to true if checkout should be done with submodules.
Attributes inherited from Importer
#interactive, #options, #post_hooks, #repository_id, #source_id
Class Method Summary collapse
-
.at_least_version(*version) ⇒ Boolean
Tests the git version.
-
.can_handle?(path) ⇒ Boolean
Tests whether the given directory is a git repository.
-
.compare_versions(actual, required) ⇒ Integer
Helper method to compare two (partial) versions represented as array of integers.
- .default_config ⇒ Object
-
.git_dir(package, require_working_copy) ⇒ Object
private
(see Git#git_dir).
-
.has_uncommitted_changes?(package, with_untracked_files = false) ⇒ Boolean
private
Tests whether the package's working copy has uncommitted changes.
- .lfs_installed? ⇒ Boolean
-
.resolve_git_dir(path) ⇒ (String,Symbol)?
private
Resolves the git directory associated with path, and tells whether it is a bare repository or not.
-
.run_git(package, *args) ⇒ Object
private
(see Git#run_git).
-
.run_git_bare(package, *args) ⇒ Object
private
(see Git#run_git_bare).
-
.validate_git_dir(package, require_working_copy, _dir, style) ⇒ void
private
Validates the return value of Git.resolve_git_dir.
-
.vcs_definition_for(path, remote_name = 'autobuild') ⇒ Object
Returns a hash that represents the configuration of a git importer based on the information contained in the git configuration.
-
.version ⇒ String
Returns the git version as a string.
Instance Method Summary collapse
- #checkout(package, _options = Hash.new) ⇒ Object
-
#commit_present_in?(package, rev, reference) ⇒ Boolean
Tests whether a commit is already present in a given history.
-
#current_branch(package) ⇒ String?
Returns the branch HEAD is pointing to.
-
#current_remote_commit(package, options = Hash.new) ⇒ String
private
Returns the commit ID of what we should consider being the remote commit.
- #declare_alternate_repository(name, repository, options = Hash.new) ⇒ Object
-
#delta_between_tags(package, from_tag, to_tag) ⇒ Object
Computes the merge status for this package between two existing tags.
-
#describe_commit_on_remote(package, rev = 'HEAD', options = Hash.new) ⇒ String
Finds a remote reference that contains a commit.
-
#describe_rev(package, rev) ⇒ Object
Resolve a commit ref to a tag or commit ID.
- #detached_head?(package) ⇒ Boolean
- #determine_target_state(package, only_local: false) ⇒ Object
- #each_alternate_path(package) ⇒ Object
-
#each_configured_remote {|remote_name, url, push_to| ... } ⇒ Object
Enumerates the remotes that this importer would set up on the repository.
-
#each_remote_ref(package) {|ref_name, commit_id| ... } ⇒ Object
Enumerates the ref that are present on the remote.
-
#fetch_remote(package, options = Hash.new) ⇒ Object
private
Fetches updates from the remote repository.
-
#git_dir(package, require_working_copy) ⇒ Object
private
Returns either the package's working copy or git directory.
-
#has_all_branches? ⇒ Boolean
Whether both the local and remote branches are known.
- #has_branch?(package, branch_name) ⇒ Boolean
- #has_commit?(package, commit_id) ⇒ Boolean
- #has_local_branch?(package) ⇒ Boolean
-
#initialize(repository, branch = nil, options = {}) ⇒ Git
constructor
Creates an importer which tracks a repository and branch.
-
#merge? ⇒ Boolean
True if it is allowed to merge remote updates automatically.
- #merge_if_simple(package, target_commit) ⇒ Object private
-
#merge_status(package, fetch_commit, reference_commit = "HEAD") ⇒ Object
Computes the update status to update a branch whose tip is at reference_commit (which can be a symbolic reference) using the fetch_commit commit.
-
#on_local_branch?(package) ⇒ Boolean
Checks if the current branch is the target branch.
-
#on_target_branch?(package) ⇒ Boolean
deprecated
Deprecated.
use on_local_branch? instead
-
#relocate(repository, options = Hash.new) ⇒ Object
Changes the repository this importer is pointing to.
-
#reset_head_to_commit(package, target_commit, fetch_commit, options = Hash.new) ⇒ Object
private
Safely resets the current branch to a given commit.
-
#resolve_all_branches(package, only_local: false) ⇒ Object
Resolve branches based on the remote's HEAD.
-
#resolve_remote_head(package, only_local: false) ⇒ Object
Return default local branch if exists, if not return the default remote branch.
-
#rev_parse(package, name, object_type = "commit") ⇒ String
private
Resolves a revision into a commit ID.
-
#run_git(package, *args) ⇒ Object
private
Run a git command that require a working copy.
-
#run_git_bare(package, *args) ⇒ Object
private
Run a git command that only need a git directory.
-
#setup_remote(package, remote_name, repository, push_to = repository) ⇒ Object
private
Set a remote up in the repositorie's configuration.
-
#show(package, commit, path) ⇒ String
Returns the file's conents at a certain commit.
-
#single_branch? ⇒ Boolean
Whether 'clone' should fetch only the remote branch, or all the branches.
-
#status(package, options = Hash.new) ⇒ Object
Returns a Status object that represents the status of this package w.r.t.
-
#tags(package, options = Hash.new) ⇒ Hash<String,String>
The tags of this packages.
-
#try_resolve_remote_head_from_local(package) ⇒ Object
Return the remote head branch from local copy if exists, if not return nil.
-
#try_resolve_remote_head_from_server(package) ⇒ Object
Return the remote head branch from server if exists, if not return 'master'.
- #update(package, options = Hash.new) ⇒ Object
-
#update_alternates(package) ⇒ void
private
Updates the git alternates file in the already checked out package to match #alternates.
-
#update_remotes_configuration(package) ⇒ Object
private
Updates the git repository's configuration for the target remote.
- #uses_lfs?(package) ⇒ Boolean
-
#validate_importdir(package) ⇒ Object
private
Verifies that the package's Package#importdir points to a git repository.
- #vcs_fingerprint(package) ⇒ Object
-
#with_submodules? ⇒ Boolean
Whether the git checkout should be done with submodules.
Methods inherited from Importer
add_post_hook, #add_post_hook, #apply, cache_dirs, #call_patch, #currently_applied_patches, default_cache_dirs, default_cache_dirs=, each_post_hook, #each_post_hook, #execute_post_hooks, fallback, #fallback, #fingerprint, #import, #interactive?, #parse_patch_list, #patch, #patchdir, #patches, #patches_fingerprint, #patchlist, #perform_checkout, #perform_update, #retry_count, #retry_count=, #save_patch_state, set_cache_dirs, #supports_relocation?, #unapply, unset_cache_dirs, #update_retry_count
Constructor Details
#initialize(repository, branch = nil, options = {}) ⇒ Git
121 122 123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 163 164 165 166 167 168 169 170 171 172 173 174 175 176 |
# File 'lib/autobuild/import/git.rb', line 121 def initialize(repository, branch = nil, = {}) @git_dir_cache = Array.new @local_branch = @remote_branch = nil @tag = @commit = nil @merge = false if branch.respond_to?(:to_hash) = branch.to_hash branch = nil end if branch Autobuild.warn "the git importer now expects you to provide the branch "\ "as a named option" Autobuild.warn "this form is deprecated:" Autobuild.warn "Autobuild.git 'git://gitorious.org/rock/buildconf.git'," Autobuild.warn " 'master'" Autobuild.warn "and should be replaced by" Autobuild.warn "Autobuild.git 'git://gitorious.org/rock/buildconf.git'," Autobuild.warn " branch: 'master'" end gitopts, common = Kernel. , push_to: nil, branch: nil, local_branch: nil, remote_branch: nil, tag: nil, commit: nil, repository_id: nil, source_id: nil, with_submodules: false, single_branch: false if gitopts[:branch] && branch raise ConfigException, "git branch specified with both the option hash "\ "and the explicit parameter" end gitopts[:branch] ||= branch super(common) @single_branch = gitopts[:single_branch] @with_submodules = gitopts.delete(:with_submodules) @alternates = if @with_submodules [] else Git.default_alternates.dup end @remote_name = 'autobuild' @push_to = nil relocate(repository, gitopts) @additional_remotes = Array.new end |
Class Attribute Details
.default_alternates ⇒ Array
A default list of repositories that should be used as reference repositories for all Git importers
It is initialized (by order of priority) using the AUTOBUILD_GIT_CACHE_DIR and AUTOBUILD_CACHE_DIR environment variables
Because of its role within the caching system in autobuild/autoproj, these defaults are not applied to git repositories that are using submodules. The autoproj cache builder does not generate repositories compatible with having submodules
37 38 39 40 41 42 43 44 45 |
# File 'lib/autobuild/import/git.rb', line 37 def default_alternates if @default_alternates then @default_alternates elsif (cache_dirs = Importer.cache_dirs('git')) @default_alternates = cache_dirs.map do |path| File.join(File.(path), '%s') end else Array.new end end |
Instance Attribute Details
#additional_remotes ⇒ (String,String,String) (readonly)
A list of remotes that should be set up in the git config
Use #declare_alternate_repository to add one
239 240 241 |
# File 'lib/autobuild/import/git.rb', line 239 def additional_remotes @additional_remotes end |
#alternates ⇒ Array<String>
A list of local (same-host) repositories that will be used instead of the remote one when possible. It has one major issue (see below), so use at your own risk.
The paths must point to the git directory, so either the .git directory in a checked out git repository, or the repository itself in a bare repository.
A default reference repository can be given through the AUTOBUILD_GIT_CACHE environment variable.
Note that it has the major caveat that if objects disappear from the reference repository, the current one will be broken. See the git documentation for more information.
231 232 233 |
# File 'lib/autobuild/import/git.rb', line 231 def alternates @alternates end |
#branch ⇒ Object
The branch this importer is tracking
207 208 209 |
# File 'lib/autobuild/import/git.rb', line 207 def branch @branch end |
#commit ⇒ Object
The commit we are pointing to. It is a commit ID.
Setting it through this method is deprecated, use #relocate to set the commit
265 266 267 |
# File 'lib/autobuild/import/git.rb', line 265 def commit @commit end |
#local_branch ⇒ Object
The branch that should be used on the local clone
Defaults to #branch
244 245 246 |
# File 'lib/autobuild/import/git.rb', line 244 def local_branch @local_branch || branch end |
#merge=(value) ⇒ Object (writeonly)
Set the merge flag. See #merge?
275 276 277 |
# File 'lib/autobuild/import/git.rb', line 275 def merge=(value) @merge = value end |
#push_to ⇒ Object
If set, this URL will be listed as a pushurl for the tracked branch. It makes it possible to have a read-only URL for fetching and specify a push URL for people that have commit rights
It is not used by the importer itself
#repository is always used for read-only operations
199 200 201 |
# File 'lib/autobuild/import/git.rb', line 199 def push_to @push_to end |
#remote_branch ⇒ Object
The remote branch to which we should push
Defaults to #branch
251 252 253 |
# File 'lib/autobuild/import/git.rb', line 251 def remote_branch @remote_branch || branch end |
#remote_name ⇒ Object
The name of the remote that should be set up by the importer
Defaults to 'autobuild'
185 186 187 |
# File 'lib/autobuild/import/git.rb', line 185 def remote_name @remote_name end |
#repository ⇒ Object
The remote repository URL.
190 191 192 |
# File 'lib/autobuild/import/git.rb', line 190 def repository @repository end |
#single_branch=(value) ⇒ Object (writeonly)
Set the #single_branch? predicate
289 290 291 |
# File 'lib/autobuild/import/git.rb', line 289 def single_branch=(value) @single_branch = value end |
#tag ⇒ Object
The tag we are pointing to. It is a tag name.
Setting it through this method is deprecated, use #relocate to set the tag
259 260 261 |
# File 'lib/autobuild/import/git.rb', line 259 def tag @tag end |
#with_submodules=(value) ⇒ Object (writeonly)
Set to true if checkout should be done with submodules
Defaults to false
204 205 206 |
# File 'lib/autobuild/import/git.rb', line 204 def with_submodules=(value) @with_submodules = value end |
Class Method Details
.at_least_version(*version) ⇒ Boolean
Tests the git version
90 91 92 |
# File 'lib/autobuild/import/git.rb', line 90 def self.at_least_version(*version) compare_versions(self.version, version) <= 0 end |
.can_handle?(path) ⇒ Boolean
Tests whether the given directory is a git repository
1316 1317 1318 1319 |
# File 'lib/autobuild/import/git.rb', line 1316 def self.can_handle?(path) _, style = Git.resolve_git_dir(path) style == :normal end |
.compare_versions(actual, required) ⇒ Integer
Helper method to compare two (partial) versions represented as array of integers
73 74 75 76 77 78 79 80 81 82 83 |
# File 'lib/autobuild/import/git.rb', line 73 def self.compare_versions(actual, required) return -compare_versions(required, actual) if actual.size > required.size actual += [0] * (required.size - actual.size) actual.zip(required).each do |v_act, v_req| if v_act > v_req then return -1 elsif v_act < v_req then return 1 end end 0 end |
.default_config ⇒ Object
48 49 50 |
# File 'lib/autobuild/import/git.rb', line 48 def self.default_config @default_config ||= Hash.new end |
.git_dir(package, require_working_copy) ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
(see Git#git_dir)
406 407 408 409 410 |
# File 'lib/autobuild/import/git.rb', line 406 def self.git_dir(package, require_working_copy) dir, style = Git.resolve_git_dir(package.importdir) validate_git_dir(package, require_working_copy, dir, style) dir end |
.has_uncommitted_changes?(package, with_untracked_files = false) ⇒ Boolean
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Tests whether the package's working copy has uncommitted changes
669 670 671 672 673 674 675 676 |
# File 'lib/autobuild/import/git.rb', line 669 def self.has_uncommitted_changes?(package, with_untracked_files = false) status = run_git(package, 'status', '--porcelain').map(&:strip) if with_untracked_files !status.empty? else status.any? { |l| l[0, 2] !~ /^\?\?|^ / } end end |
.lfs_installed? ⇒ Boolean
1242 1243 1244 1245 1246 1247 |
# File 'lib/autobuild/import/git.rb', line 1242 def self.lfs_installed? return @lfs_installed unless @lfs_installed.nil? _, _, status = Open3.capture3('git lfs') @lfs_installed = status.success? end |
.resolve_git_dir(path) ⇒ (String,Symbol)?
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Resolves the git directory associated with path, and tells whether it is a bare repository or not
367 368 369 370 371 372 373 374 375 376 377 378 379 380 |
# File 'lib/autobuild/import/git.rb', line 367 def self.resolve_git_dir(path) gitdir = File.join(path, '.git') path = gitdir if File.exist?(gitdir) result = `#{Autobuild.tool(:git)} --git-dir="#{path}" rev-parse \ --is-bare-repository 2>&1` if $CHILD_STATUS.success? if result.strip == "true" [path, :bare] else [path, :normal] end end end |
.run_git(package, *args) ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
(see Git#run_git)
500 501 502 503 504 505 506 507 508 509 510 511 512 |
# File 'lib/autobuild/import/git.rb', line 500 def self.run_git(package, *args) = if args.last.kind_of?(Hash) args.pop else Hash.new end working_directory = File.dirname(git_dir(package, true)) package.run(:import, Autobuild.tool(:git), *args, Hash[resolved_env: Hash.new, working_directory: working_directory].merge()) end |
.run_git_bare(package, *args) ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
(see Git#run_git_bare)
526 527 528 529 530 531 532 533 534 535 536 537 |
# File 'lib/autobuild/import/git.rb', line 526 def self.(package, *args) = if args.last.kind_of?(Hash) args.pop else Hash.new end package.run(:import, Autobuild.tool(:git), '--git-dir', git_dir(package, false), *args, Hash[resolved_env: Hash.new].merge()) end |
.validate_git_dir(package, require_working_copy, _dir, style) ⇒ void
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
This method returns an undefined value.
Validates the return value of resolve_git_dir
429 430 431 432 433 434 435 436 437 438 439 |
# File 'lib/autobuild/import/git.rb', line 429 def self.validate_git_dir(package, require_working_copy, _dir, style) if !style raise ConfigException.new(package, 'import', retry: false), "while importing #{package.name}, #{package.importdir} "\ "does not point to a git repository" elsif require_working_copy && (style == :bare) raise ConfigException.new(package, 'import', retry: false), "while importing #{package.name}, #{package.importdir} "\ "points to a bare git repository but a working copy was required" end end |
.vcs_definition_for(path, remote_name = 'autobuild') ⇒ Object
Returns a hash that represents the configuration of a git importer based on the information contained in the git configuration
1325 1326 1327 1328 1329 1330 1331 1332 1333 1334 1335 1336 1337 1338 1339 1340 1341 1342 1343 1344 1345 |
# File 'lib/autobuild/import/git.rb', line 1325 def self.vcs_definition_for(path, remote_name = 'autobuild') unless can_handle?(path) raise ArgumentError, "#{path} is neither a git repository, "\ "nor a bare git repository" end Dir.chdir(path) do vars = `#{Autobuild.tool(:git)} config -l`. split("\n"). each_with_object(Hash.new) do |line, h| k, v = line.strip.split('=', 2) h[k] = v end url = vars["remote.#{remote_name}.url"] || vars['remote.origin.url'] if url return Hash[type: :git, url: url] else return Hash[type: :git] end end end |
.version ⇒ String
Returns the git version as a string
57 58 59 60 61 62 63 64 65 66 |
# File 'lib/autobuild/import/git.rb', line 57 def self.version version = Subprocess.run('git', 'setup', Autobuild.tool(:git), '--version'). first if version =~ /^git version (\d[\d.]+)/ $1.split(".").map { |i| Integer(i) } else raise ArgumentError, "cannot parse git version string #{version}, "\ "was expecting something looking like 'git version 2.1.0'" end end |
Instance Method Details
#checkout(package, _options = Hash.new) ⇒ Object
1249 1250 1251 1252 1253 1254 1255 1256 1257 1258 1259 1260 1261 1262 1263 1264 1265 1266 1267 1268 1269 1270 1271 1272 1273 1274 1275 1276 1277 1278 |
# File 'lib/autobuild/import/git.rb', line 1249 def checkout(package, = Hash.new) base_dir = File.('..', package.importdir) FileUtils.mkdir_p(base_dir) unless File.directory?(base_dir) = Array.new << '--recurse-submodules' if with_submodules? if single_branch? if remote_branch.start_with?("refs/") raise ArgumentError, "you cannot provide a full ref for"\ " the remote branch while cloning a single branch" end << "--branch=#{remote_branch}" << "--single-branch" end each_alternate_path(package) do |path| << '--reference' << path end self.class.default_config.each do |key, value| << "--config=#{key}=#{value}" end package.run(:import, Autobuild.tool('git'), 'clone', '-o', remote_name, *, repository, package.importdir, retry: true) update_remotes_configuration(package) update(package, only_local: !remote_branch.start_with?("refs/"), reset: :force) if with_submodules? run_git(package, "submodule", "update", '--init', '--recursive') end end |
#commit_present_in?(package, rev, reference) ⇒ Boolean
Tests whether a commit is already present in a given history
894 895 896 897 898 899 900 901 902 903 904 |
# File 'lib/autobuild/import/git.rb', line 894 def commit_present_in?(package, rev, reference) commit = rev_parse(package, rev) begin merge_base = (package, 'merge-base', commit, reference).first merge_base == commit rescue Exception raise PackageException.new(package, 'import'), "failed to find "\ "the merge-base between #{rev} and #{reference}. "\ "Are you sure these commits exist ?" end end |
#current_branch(package) ⇒ String?
Returns the branch HEAD is pointing to
800 801 802 803 804 |
# File 'lib/autobuild/import/git.rb', line 800 def current_branch(package) (package, 'symbolic-ref', 'HEAD', '-q').first.strip rescue SubcommandFailed => e raise if e.status != 1 end |
#current_remote_commit(package, options = Hash.new) ⇒ String
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Returns the commit ID of what we should consider being the remote commit
691 692 693 694 695 696 697 698 699 700 701 702 703 704 705 706 707 708 709 710 711 712 713 714 715 716 717 718 719 720 721 722 723 |
# File 'lib/autobuild/import/git.rb', line 691 def current_remote_commit(package, = Hash.new) = Hash[only_local: ] unless .kind_of?(Hash) only_local = .delete(:only_local) if only_local unless remote_branch.start_with?("refs/") refspec = [:refspec] || ("refs/tags/#{tag}" if tag) || "refs/remotes/#{remote_name}/#{remote_branch}" end unless (refspec = Array(refspec).first) raise ArgumentError, "cannot use only_local with no tag,"\ " and an absolute remote ref" end begin (package, 'show-ref', '-s', refspec).first.strip rescue SubcommandFailed raise PackageException.new(package, "import"), "cannot resolve #{refspec}" end else refspec = [:refspec] || ("refs/tags/#{tag}" if tag) || remote_branch_to_ref(remote_branch) begin fetch_remote(package, refspec: refspec) rescue Exception => e fallback(e, package, :status, package, only_local) end end end |
#declare_alternate_repository(name, repository, options = Hash.new) ⇒ Object
1347 1348 1349 1350 1351 1352 1353 |
# File 'lib/autobuild/import/git.rb', line 1347 def declare_alternate_repository(name, repository, = Hash.new) unless name raise ArgumentError, "cannot declare alternate repository "\ "#{repository} without a name" end additional_remotes << [name, repository, [:push_to] || repository] end |
#delta_between_tags(package, from_tag, to_tag) ⇒ Object
Computes the merge status for this package between two existing tags
447 448 449 450 451 452 453 454 455 456 457 458 459 460 461 462 |
# File 'lib/autobuild/import/git.rb', line 447 def (package, from_tag, to_tag) = (package) unless .key?(from_tag) raise ArgumentError, "tag '#{from_tag}' is unknown to #{package.name} "\ "-- known tags are: #{.keys}" end unless .key?(to_tag) raise ArgumentError, "tag '#{to_tag}' is unknown to #{package.name} "\ "-- known tags are: #{.keys}" end from_commit = [from_tag] to_commit = [to_tag] merge_status(package, to_commit, from_commit) end |
#describe_commit_on_remote(package, rev = 'HEAD', options = Hash.new) ⇒ String
Finds a remote reference that contains a commit
It will favor the configured #remote_branch if it matches
914 915 916 917 918 919 920 921 922 923 924 925 926 927 928 929 930 931 932 933 934 935 936 937 938 939 940 941 942 943 944 945 946 947 948 949 950 951 952 953 954 955 956 957 958 959 960 961 962 963 964 965 |
# File 'lib/autobuild/import/git.rb', line 914 def describe_commit_on_remote(package, rev = 'HEAD', = Hash.new) rev = rev.to_str = Kernel. , tags: true commit_id = rev_parse(package, rev) remote_refs = Hash[*each_remote_ref(package).to_a.flatten] remote_branch_ref = remote_branch_to_ref(remote_branch) remote_branch_id = remote_refs.delete(remote_branch_ref) begin if commit_present_in?(package, commit_id, remote_branch_id) return remote_branch end rescue PackageException # We have to fetch. Fetch all branches at once fetch_remote(package, refspec: [remote_branch_ref, *remote_refs.keys]) if commit_present_in?(package, commit_id, remote_branch_id) return remote_branch end end remote_refs.delete_if { |r| r =~ %r{^refs/tags/} } unless [:tags] # Prefer tags, then heads, then the rest (e.g. github pull requests) remote_refs = remote_refs.sort_by do |rev_name, _rev_id| case rev_name when %r{^refs/tags/} then 0 when %r{^refs/heads/} then 1 else 2 end end remote_refs.delete_if do |rev_name, rev_id| if commit_present_in?(package, commit_id, rev_id) return rev_name else true end rescue PackageException false end unless remote_refs.empty? fetch_remote(package, refspec: remote_refs.map(&:first)) remote_refs.each do |rev_name, rev_id| return rev_name if commit_present_in?(package, commit_id, rev_id) end end raise PackageException.new(package), "current HEAD (#{commit_id}) does not "\ "seem to be present on the remote" end |
#describe_rev(package, rev) ⇒ Object
Resolve a commit ref to a tag or commit ID
594 595 596 597 598 599 600 601 |
# File 'lib/autobuild/import/git.rb', line 594 def describe_rev(package, rev) tag = (package, 'describe', '--tags', '--exact-match', rev).first.strip [true, tag.encode('UTF-8')] rescue Autobuild::SubcommandFailed commit = rev_parse(package, rev) [false, commit.encode('UTF-8')] end |
#detached_head?(package) ⇒ Boolean
781 782 783 |
# File 'lib/autobuild/import/git.rb', line 781 def detached_head?(package) current_branch(package).nil? end |
#determine_target_state(package, only_local: false) ⇒ Object
1110 1111 1112 1113 1114 1115 1116 1117 1118 1119 1120 1121 1122 1123 1124 1125 1126 1127 1128 1129 1130 |
# File 'lib/autobuild/import/git.rb', line 1110 def determine_target_state(package, only_local: false) pinned_state = if commit then commit elsif tag then "refs/tags/#{tag}" end if pinned_state unless has_commit?(package, pinned_state) fetch_commit = current_remote_commit( package, only_local: only_local, refspec: [remote_branch_to_ref(remote_branch), tag]) end target_commit = pinned_state = rev_parse(package, pinned_state) else target_commit = fetch_commit = current_remote_commit(package, only_local: only_local) end [pinned_state, target_commit, fetch_commit] end |
#each_alternate_path(package) ⇒ Object
1222 1223 1224 1225 1226 1227 1228 1229 1230 |
# File 'lib/autobuild/import/git.rb', line 1222 def each_alternate_path(package) return enum_for(__method__, package) unless block_given? alternates.each do |path| path = format(path, package.name) yield(path) if File.directory?(path) end nil end |
#each_configured_remote {|remote_name, url, push_to| ... } ⇒ Object
Enumerates the remotes that this importer would set up on the repository
570 571 572 573 574 |
# File 'lib/autobuild/import/git.rb', line 570 def each_configured_remote ([['autobuild', repository, push_to]] + additional_remotes).each do |args| yield(args[0], args[1], args[2] || args[1]) end end |
#each_remote_ref(package) {|ref_name, commit_id| ... } ⇒ Object
Enumerates the ref that are present on the remote
607 608 609 610 611 612 613 614 |
# File 'lib/autobuild/import/git.rb', line 607 def each_remote_ref(package) return enum_for(__method__, package) unless block_given? (package, 'ls-remote', repository).each do |line| commit_id, ref_name = line.split(/\s+/) yield(ref_name, commit_id) if ref_name !~ /\^/ end end |
#fetch_remote(package, options = Hash.new) ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Fetches updates from the remote repository. Returns the remote commit ID on success, nil on failure. Expects the current directory to be the package's source directory.
621 622 623 624 625 626 627 628 629 630 631 632 633 634 635 636 637 638 639 640 641 642 643 644 645 646 647 648 649 650 651 652 653 654 655 656 657 658 659 660 |
# File 'lib/autobuild/import/git.rb', line 621 def fetch_remote(package, = Hash.new) validate_importdir(package) unless [:refspec] raise ArgumentError, "required argument 'refspec' not given" end git_dir = git_dir(package, false) # If we are checking out a specific commit, we don't know which # branch to refer to in git fetch. So, we have to set up the # remotes and call git fetch directly (so that all branches get # fetch) # # Otherwise, do git fetch now # # Doing it now is better as it makes sure that we replace the # configuration parameters only if the repository and branch are # OK (i.e. we keep old working configuration instead) refspec = Array([:refspec]) tag_arg = ['--tags'] if tag (package, 'fetch', repository, *tag_arg, *refspec, retry: true) update_remotes_configuration(package) # Now get the actual commit ID from the FETCH_HEAD file, and # return it if File.readable?(File.join(git_dir, 'FETCH_HEAD')) fetched_commits = File.readlines(File.join(git_dir, 'FETCH_HEAD')). find_all { |l| l !~ /not-for-merge/ }. map { |line| line.split(/\s+/).first } refspec.zip(fetched_commits).each do |spec, commit_id| if spec =~ %r{^refs/heads/(.*)$} (package, 'update-ref', "-m", "updated by autobuild", "refs/remotes/#{remote_name}/#{$1}", commit_id) end end fetched_commits.first end end |
#git_dir(package, require_working_copy) ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Returns either the package's working copy or git directory
391 392 393 394 395 396 397 398 399 400 401 |
# File 'lib/autobuild/import/git.rb', line 391 def git_dir(package, require_working_copy) if @git_dir_cache[0] == package.importdir dir, style = *@git_dir_cache[1, 2] else dir, style = Git.resolve_git_dir(package.importdir) end @git_dir_cache = [package.importdir, dir, style] self.class.validate_git_dir(package, require_working_copy, dir, style) dir end |
#has_all_branches? ⇒ Boolean
Whether both the local and remote branches are known
See documentation of #resolve_all_branches
334 335 336 |
# File 'lib/autobuild/import/git.rb', line 334 def has_all_branches? remote_branch && local_branch end |
#has_branch?(package, branch_name) ⇒ Boolean
766 767 768 769 770 771 772 773 774 775 |
# File 'lib/autobuild/import/git.rb', line 766 def has_branch?(package, branch_name) (package, 'show-ref', '-q', '--verify', remote_branch_to_ref(branch_name)) true rescue SubcommandFailed => e if e.status == 1 false else raise end end |
#has_commit?(package, commit_id) ⇒ Boolean
756 757 758 759 760 761 762 763 764 |
# File 'lib/autobuild/import/git.rb', line 756 def has_commit?(package, commit_id) (package, 'rev-parse', '-q', '--verify', "#{commit_id}^{commit}") true rescue SubcommandFailed => e if e.status == 1 false else raise end end |
#has_local_branch?(package) ⇒ Boolean
777 778 779 |
# File 'lib/autobuild/import/git.rb', line 777 def has_local_branch?(package) has_branch?(package, local_branch) if local_branch end |
#merge? ⇒ Boolean
True if it is allowed to merge remote updates automatically. If false (the default), the import will fail if the updates do not resolve as a fast-forward
270 271 272 |
# File 'lib/autobuild/import/git.rb', line 270 def merge? @merge end |
#merge_if_simple(package, target_commit) ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
1195 1196 1197 1198 1199 1200 1201 1202 1203 1204 1205 1206 1207 1208 1209 |
# File 'lib/autobuild/import/git.rb', line 1195 def merge_if_simple(package, target_commit) status = merge_status(package, target_commit) if status.needs_update? if !merge? && status.status == Status::NEEDS_MERGE raise PackageException.new(package, 'import'), "the local branch "\ "'#{local_branch}' and the remote branch #{branch} of "\ "#{package.name} have diverged, and I therefore refuse "\ "to update automatically. Go into #{package.importdir} "\ "and either reset the local branch or merge the remote changes" end run_git(package, 'merge', target_commit) return true end false end |
#merge_status(package, fetch_commit, reference_commit = "HEAD") ⇒ Object
Computes the update status to update a branch whose tip is at reference_commit (which can be a symbolic reference) using the fetch_commit commit
I.e. this compute what happens if one would do
git checkout reference_commit
git merge fetch_commit
976 977 978 979 980 981 982 983 984 985 986 987 988 989 990 991 992 993 994 995 996 997 998 999 1000 1001 |
# File 'lib/autobuild/import/git.rb', line 976 def merge_status(package, fetch_commit, reference_commit = "HEAD") begin common_commit = (package, 'merge-base', reference_commit, fetch_commit).first.strip rescue Exception raise PackageException.new(package, 'import'), "failed to find "\ "the merge-base between #{reference_commit} and #{fetch_commit}. "\ "Are you sure these commits exist ?" end remote_commit = rev_parse(package, fetch_commit) head_commit = rev_parse(package, reference_commit) status = if common_commit != remote_commit if common_commit == head_commit Status::SIMPLE_UPDATE else Status::NEEDS_MERGE end elsif common_commit == head_commit Status::UP_TO_DATE else Status::ADVANCED end Status.new(package, status, fetch_commit, head_commit, common_commit) end |
#on_local_branch?(package) ⇒ Boolean
Checks if the current branch is the target branch. Expects that the current directory is the package's directory
808 809 810 811 812 |
# File 'lib/autobuild/import/git.rb', line 808 def on_local_branch?(package) if (current_branch = self.current_branch(package)) current_branch == "refs/heads/#{local_branch}" end end |
#on_target_branch?(package) ⇒ Boolean
use on_local_branch? instead
815 816 817 |
# File 'lib/autobuild/import/git.rb', line 815 def on_target_branch?(package) on_local_branch?(package) end |
#relocate(repository, options = Hash.new) ⇒ Object
Changes the repository this importer is pointing to
1281 1282 1283 1284 1285 1286 1287 1288 1289 1290 1291 1292 1293 1294 1295 1296 1297 1298 1299 1300 1301 1302 1303 1304 1305 1306 1307 1308 1309 1310 1311 1312 1313 |
# File 'lib/autobuild/import/git.rb', line 1281 def relocate(repository, = Hash.new) = .transform_keys(&:to_sym) local_branch = [:local_branch] || [:branch] || self.local_branch || [:default_branch] || nil remote_branch = [:remote_branch] || [:branch] || self.remote_branch || [:default_branch] || nil if local_branch&.start_with?("refs/") raise ArgumentError, "you cannot provide a full ref for"\ " the local branch, only for the remote branch" end @push_to = [:push_to] || @push_to @branch = @local_branch = @remote_branch = nil if local_branch == remote_branch @branch = local_branch else @local_branch = local_branch @remote_branch = remote_branch end @tag = .fetch(:tag, @tag) @commit = .fetch(:commit, @commit) @repository = repository.to_str @repository_id = [:repository_id] || "git:#{@repository}" @source_id = [:source_id] || "#{@repository_id} branch=#{remote_branch} tag=#{tag} commit=#{commit}" end |
#reset_head_to_commit(package, target_commit, fetch_commit, options = Hash.new) ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Safely resets the current branch to a given commit
This method safely resets the current branch to a given commit, not requiring a clean working copy (i.e. it can handle local changes).
It verifies that the current HEAD will not be lost by the operation, either because it is included in the target commit or because it is present remotely
1062 1063 1064 1065 1066 1067 1068 1069 1070 1071 1072 1073 1074 1075 1076 1077 1078 1079 1080 1081 1082 1083 1084 1085 1086 1087 1088 1089 1090 1091 1092 1093 1094 1095 1096 1097 1098 1099 1100 1101 1102 1103 1104 1105 1106 1107 1108 |
# File 'lib/autobuild/import/git.rb', line 1062 def reset_head_to_commit(package, target_commit, fetch_commit, = Hash.new) current_head = rev_parse(package, 'HEAD') head_to_target = merge_status(package, target_commit, current_head) status_to_target = head_to_target.status if status_to_target == Status::UP_TO_DATE return false elsif status_to_target == Status::SIMPLE_UPDATE run_git(package, 'merge', target_commit) elsif ![:force] # Check whether the current HEAD is present on the remote # repository. We'll refuse resetting if there are uncommitted # changes unless commit_present_in?(package, current_head, fetch_commit) raise ImporterCannotReset.new(package, 'import'), "branch #{local_branch} of #{package.name} contains"\ " commits that do not seem to be present on the branch"\ " #{remote_branch} of the remote repository. I can't"\ " go on as it could make you lose some stuff. Update"\ " the remote branch in your overrides, push your"\ " changes or reset to the remote commit manually"\ " before trying again" end end package. format(" %%s: resetting branch %<branch>s to %<commit>s", branch: local_branch, commit: target_commit) # I don't use a reset --hard here as it would add even more # restrictions on when we can do the operation (as we would refuse # doing it if there are local changes). The checkout creates a # detached HEAD, but makes sure that applying uncommitted changes is # fine (it would abort otherwise). The rest then updates HEAD and # the local_branch ref to match the required target commit resolved_target_commit = rev_parse(package, "#{target_commit}^{commit}") begin run_git(package, 'checkout', target_commit) run_git(package, 'update-ref', "refs/heads/#{local_branch}", resolved_target_commit) run_git(package, 'symbolic-ref', "HEAD", "refs/heads/#{local_branch}") rescue ::Exception run_git(package, 'symbolic-ref', "HEAD", target_commit) run_git(package, 'update-ref', "refs/heads/#{local_branch}", current_head) run_git(package, 'checkout', "refs/heads/#{local_branch}") raise end true end |
#resolve_all_branches(package, only_local: false) ⇒ Object
Resolve branches based on the remote's HEAD
Since GitHub (and others) decided to change the name of the “default” branch, we can't assume that master is … well … master.
For this reason, a Git importer does not have a built-in default. If the branch(es) are not provided explicitly, the importer will call this method to guess the name of the default branch instead.
Call #has_all_branches? to determine whether it is necessary
348 349 350 351 352 353 354 355 356 357 |
# File 'lib/autobuild/import/git.rb', line 348 def resolve_all_branches(package, only_local: false) default_branch = resolve_remote_head(package, only_local: only_local) unless default_branch raise NetworkAccessNeeded, "determining the remote branch would require access to "\ "the network, and only_local is true" end relocate(repository, default_branch: default_branch) end |
#resolve_remote_head(package, only_local: false) ⇒ Object
Return default local branch if exists, if not return the default remote branch
326 327 328 329 |
# File 'lib/autobuild/import/git.rb', line 326 def resolve_remote_head(package, only_local: false) try_resolve_remote_head_from_local(package) || (!only_local && try_resolve_remote_head_from_server(package)) end |
#rev_parse(package, name, object_type = "commit") ⇒ String
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Resolves a revision into a commit ID
864 865 866 867 868 869 870 871 |
# File 'lib/autobuild/import/git.rb', line 864 def rev_parse(package, name, object_type = "commit") name = "#{name}^{#{object_type}}" if object_type (package, 'rev-parse', '-q', '--verify', name).first rescue Autobuild::SubcommandFailed raise PackageException.new(package, 'import'), "failed to resolve #{name}. "\ "Are you sure this commit, branch or tag exists ?" end |
#run_git(package, *args) ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Run a git command that require a working copy
493 494 495 |
# File 'lib/autobuild/import/git.rb', line 493 def run_git(package, *args) self.class.run_git(package, *args) end |
#run_git_bare(package, *args) ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Run a git command that only need a git directory
519 520 521 |
# File 'lib/autobuild/import/git.rb', line 519 def (package, *args) self.class.(package, *args) end |
#setup_remote(package, remote_name, repository, push_to = repository) ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Set a remote up in the repositorie's configuration
542 543 544 545 546 547 548 549 550 551 552 553 554 555 556 557 558 559 560 561 562 |
# File 'lib/autobuild/import/git.rb', line 542 def setup_remote(package, remote_name, repository, push_to = repository) resolve_all_branches(package, only_local: true) unless has_all_branches? (package, 'config', '--replace-all', "remote.#{remote_name}.url", repository) (package, 'config', '--replace-all', "remote.#{remote_name}.pushurl", push_to || repository) (package, 'config', '--replace-all', "remote.#{remote_name}.fetch", "+refs/heads/*:refs/remotes/#{remote_name}/*") if remote_branch && local_branch remote_ref = remote_branch_to_ref(remote_branch) (package, 'config', '--replace-all', "remote.#{remote_name}.push", "refs/heads/#{local_branch}:#{remote_ref}") else (package, 'config', '--replace-all', "remote.#{remote_name}.push", "refs/heads/*:refs/heads/*") end end |
#show(package, commit, path) ⇒ String
Returns the file's conents at a certain commit
879 880 881 882 883 884 |
# File 'lib/autobuild/import/git.rb', line 879 def show(package, commit, path) (package, 'show', "#{commit}:#{path}").join("\n") rescue Autobuild::SubcommandFailed raise PackageException.new(package, 'import'), "failed to either resolve commit #{commit} or file #{path}" end |
#single_branch? ⇒ Boolean
Whether 'clone' should fetch only the remote branch, or all the branches
284 285 286 |
# File 'lib/autobuild/import/git.rb', line 284 def single_branch? @single_branch end |
#status(package, options = Hash.new) ⇒ Object
Returns a Status object that represents the status of this package w.r.t. the expected remote repository and branch
727 728 729 730 731 732 733 734 735 736 737 738 739 740 741 742 743 744 745 746 747 748 749 750 751 752 753 754 |
# File 'lib/autobuild/import/git.rb', line 727 def status(package, = Hash.new) only_local = if .kind_of?(Hash) .fetch(:only_local, false) else end validate_importdir(package) resolve_all_branches(package, only_local: only_local) unless has_all_branches? _pinned_state, target_commit, = determine_target_state( package, only_local: only_local) status = merge_status(package, target_commit) status.uncommitted_code = self.class.has_uncommitted_changes?(package) if (current_branch = self.current_branch(package)) if current_branch != "refs/heads/#{local_branch}" status.unexpected_working_copy_state << "working copy is on branch #{current_branch}, "\ "the autoproj configuration expected it to be on #{local_branch}" end else status.unexpected_working_copy_state << "working copy is on a detached HEAD" end status end |
#tags(package, options = Hash.new) ⇒ Hash<String,String>
The tags of this packages
472 473 474 475 476 477 478 479 480 481 482 483 |
# File 'lib/autobuild/import/git.rb', line 472 def (package, = Hash.new) unless .fetch(:only_local, false) (package, 'fetch', '--tags') end tag_list = (package, 'show-ref', '--tags').map(&:strip) = Hash.new tag_list.each do |entry| commit_to_tag = entry.split(" ") [commit_to_tag[1].sub("refs/tags/", "")] = commit_to_tag[0] end end |
#try_resolve_remote_head_from_local(package) ⇒ Object
Return the remote head branch from local copy if exists, if not return nil
302 303 304 305 306 307 308 |
# File 'lib/autobuild/import/git.rb', line 302 def try_resolve_remote_head_from_local(package) ls_local_string = run_git(package, 'symbolic-ref', "refs/remotes/#{@remote_name}/HEAD").first.strip local_remote_head = ls_local_string.match("refs/remotes/#{@remote_name}/(.*)") local_remote_head ? local_remote_head[1] : nil rescue Autobuild::SubcommandFailed # rubocop:disable Lint/SuppressedException end |
#try_resolve_remote_head_from_server(package) ⇒ Object
Return the remote head branch from server if exists, if not return 'master'
313 314 315 316 317 318 319 320 321 |
# File 'lib/autobuild/import/git.rb', line 313 def try_resolve_remote_head_from_server(package) ls_remote_string = package.run( :import, Autobuild.tool('git'), 'ls-remote', '--symref', repository ).first.strip server_remote_head = ls_remote_string.match("ref:[^A-z]refs/heads/(.*)[^A-z]HEAD") server_remote_head ? server_remote_head[1] : 'master' end |
#update(package, options = Hash.new) ⇒ Object
1133 1134 1135 1136 1137 1138 1139 1140 1141 1142 1143 1144 1145 1146 1147 1148 1149 1150 1151 1152 1153 1154 1155 1156 1157 1158 1159 1160 1161 1162 1163 1164 1165 1166 1167 1168 1169 1170 1171 1172 1173 1174 1175 1176 |
# File 'lib/autobuild/import/git.rb', line 1133 def update(package, = Hash.new) validate_importdir(package) resolve_all_branches(package) unless has_all_branches? only_local = .fetch(:only_local, false) reset = .fetch(:reset, false) # This is really really a hack to workaround how broken the # importdir thing is update_alternates(package) if package.importdir == package.srcdir pinned_state, target_commit, fetch_commit = determine_target_state(package, only_local: only_local) did_change_branch = ensure_on_local_branch(package, target_commit) # Check whether we are already at the requested state if pinned_state pin_is_uptodate, pin_did_merge = handle_pinned_state(package, pinned_state, reset: reset) end unless pin_is_uptodate fetch_commit ||= current_remote_commit( package, only_local: only_local, refspec: [remote_branch_to_ref(remote_branch), tag] ) did_update = if reset reset_head_to_commit(package, target_commit, fetch_commit, force: (reset == :force)) else merge_if_simple(package, target_commit) end end if !only_local && with_submodules? run_git(package, "submodule", "update", '--init', '--recursive') did_update = true end did_update || pin_did_merge || did_change_branch end |
#update_alternates(package) ⇒ void
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
This method returns an undefined value.
Updates the git alternates file in the already checked out package to match #alternates
1010 1011 1012 1013 1014 1015 1016 1017 1018 1019 1020 1021 1022 1023 1024 1025 1026 1027 1028 1029 1030 1031 1032 1033 1034 1035 1036 1037 1038 1039 1040 1041 1042 |
# File 'lib/autobuild/import/git.rb', line 1010 def update_alternates(package) alternates_path = File.join(git_dir(package, false), 'objects', 'info', 'alternates') current_alternates = if File.file?(alternates_path) File.readlines(alternates_path) .map(&:strip) .find_all { |l| !l.empty? } else Array.new end alternates = each_alternate_path(package).map do |path| File.join(path, 'objects') end unless (current_alternates.sort - alternates.sort).empty? # Warn that something is fishy, but assume that the user knows # what he is doing package.warn "%s: the list of git alternates listed in the repository "\ "differs from the one set up in autobuild." package.warn "%s: I will update, but that is dangerous" package.warn "%s: using git alternates is for advanced users only, "\ "who know git very well." package.warn "%s: Don't complain if something breaks" end if alternates.empty? FileUtils.rm_f alternates_path else File.open(alternates_path, 'w') do |io| io.write alternates.join("\n") end end end |
#update_remotes_configuration(package) ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Updates the git repository's configuration for the target remote
579 580 581 582 583 584 585 586 587 588 589 590 591 |
# File 'lib/autobuild/import/git.rb', line 579 def update_remotes_configuration(package) each_configured_remote do |*args| setup_remote(package, *args) end if local_branch (package, 'config', '--replace-all', "branch.#{local_branch}.remote", remote_name) (package, 'config', '--replace-all', "branch.#{local_branch}.merge", remote_branch_to_ref(local_branch)) end end |
#uses_lfs?(package) ⇒ Boolean
1232 1233 1234 1235 1236 1237 1238 1239 1240 |
# File 'lib/autobuild/import/git.rb', line 1232 def uses_lfs?(package) git_files = run_git(package, 'ls-files').join("\n") git_attrs = run_git( package, 'check-attr', '--all', '--stdin', input_streams: [StringIO.new(git_files)] ).join("\n") /(.*): filter: lfs/.match(git_attrs) end |
#validate_importdir(package) ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Verifies that the package's Package#importdir points to a git repository
295 296 297 |
# File 'lib/autobuild/import/git.rb', line 295 def validate_importdir(package) git_dir(package, true) end |
#vcs_fingerprint(package) ⇒ Object
178 179 180 |
# File 'lib/autobuild/import/git.rb', line 178 def vcs_fingerprint(package) rev_parse(package, 'HEAD') end |
#with_submodules? ⇒ Boolean
Whether the git checkout should be done with submodules
278 279 280 |
# File 'lib/autobuild/import/git.rb', line 278 def with_submodules? @with_submodules end |