Method: Autobuild::Importer#post_hooks
- Defined in:
- lib/autobuild/importer.rb
#post_hooks ⇒ Object (readonly)
A list of hooks that are called after a successful checkout or update
They are added either at the instance level with #add_post_hook or globally for all importers of a given type with add_post_hook
274 275 276 |
# File 'lib/autobuild/importer.rb', line 274 def post_hooks @post_hooks end |